@@ -139,7 +139,7 @@
 
 #include "popt.h"
 
-/* "$Id: main.c,v 1.80 1999/02/23 02:53:39 unammx Exp $" */
+/* "$Id: main.c,v 1.82 1999/02/25 05:11:45 unammx Exp $" */
 
 /* When the modes are active, left_panel, right_panel and tree_panel */
 /* Point to a proper data structure.  You should check with the functions */
@@ -335,8 +335,14 @@ int boot_current_is_left = 1;
 /* Used for keeping track of the original stdout */
 int stdout_fd = 0;
 
+/*
+ * Ugh.  Pavel, you shell hack in sfs is BAD.
+ * We need to kill shell from vfs
+ */
+#ifndef USE_VFS
 /* The user's shell */
 char *shell;
+#endif
 
 /* mc_home: The home of MC */
 char *mc_home;
@@ -2520,7 +2526,7 @@ enum {
 };
 
 static void
-process_args (int c, char *option_arg)
+process_args (int c, const char *option_arg)
 {
     switch (c) {
     case 'V':
